'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.matchImageSnapshotOptions = matchImageSnapshotOptions;
exports.matchImageSnapshotResults = matchImageSnapshotResults;
exports.matchImageSnapshotPlugin = matchImageSnapshotPlugin;
exports.addMatchImageSnapshotPlugin = addMatchImageSnapshotPlugin;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _diffSnapshot = require('jest-image-snapshot/src/diff-snapshot');

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; } /**
                                                                                                                                                                                                                              * Copyright (c) 2018-present The Palmer Group
                                                                                                                                                                                                                              *
                                                                                                                                                                                                                              * This source code is licensed under the MIT license found in the
                                                                                                                                                                                                                              * LICENSE file in the root directory of this source tree.
                                                                                                                                                                                                                              */

let snapshotOptions = {};
let snapshotResults = {};
let snapshotRunning = false;
const kebabSnap = '-snap.png';
const kebabSnapFail = '-snap-FAIL.png';
const dotSnap = '.snap.png';
const dotDiff = '.diff.png';

function matchImageSnapshotOptions(options = {}) {
  snapshotOptions = options;
  snapshotRunning = true;
  return null;
}

function matchImageSnapshotResults() {
  snapshotRunning = false;
  return snapshotResults;
}

function matchImageSnapshotPlugin({ path: screenshotPath }) {
  if (!snapshotRunning) {
    return null;
  }

  const {
    screenshotsFolder,
    updateSnapshots,
    options: {
      failureThreshold = 0,
      failureThresholdType = 'pixel',
      customSnapshotsDir,
      customDiffDir
    } = {}
  } = snapshotOptions,
        options = _objectWithoutProperties(snapshotOptions.options, ['failureThreshold', 'failureThresholdType', 'customSnapshotsDir', 'customDiffDir']);

  const receivedImageBuffer = _fsExtra2.default.readFileSync(screenshotPath);
  const screenshotFileName = screenshotPath.slice(screenshotPath.lastIndexOf(_path2.default.sep) + 1);
  const screenshotDir = screenshotPath.replace(screenshotFileName, '');
  const relativePath = screenshotDir.match(/screenshots(.*)/)[1];
  const snapshotIdentifier = screenshotFileName.replace('.png', '');
  const snapshotsDir = customSnapshotsDir ? _path2.default.join(process.cwd(), customSnapshotsDir, relativePath) : _path2.default.join(screenshotsFolder, '..', 'snapshots', relativePath);

  const snapshotKebabPath = _path2.default.join(snapshotsDir, `${snapshotIdentifier}${kebabSnap}`);
  const snapshotKebabPathFail = _path2.default.join(snapshotsDir, `${snapshotIdentifier}${kebabSnapFail}`);
  const snapshotDotPath = _path2.default.join(snapshotsDir, `${snapshotIdentifier}${dotSnap}`);

  const diffDir = customDiffDir ? _path2.default.join(process.cwd(), customDiffDir, relativePath) : _path2.default.join(snapshotsDir, '__diff_output__');
  const diffDotPath = _path2.default.join(diffDir, `${snapshotIdentifier}${dotDiff}`);

  if (_fsExtra2.default.pathExistsSync(snapshotDotPath)) {
    _fsExtra2.default.copySync(snapshotDotPath, snapshotKebabPath);
  }

  snapshotResults = (0, _diffSnapshot.diffImageToSnapshot)(_extends({
    snapshotsDir,
    receivedImageBuffer,
    snapshotIdentifier,
    failureThreshold,
    failureThresholdType,
    updateSnapshot: updateSnapshots
  }, options));

  const { pass, added, updated, diffOutputPath } = snapshotResults;

  if (!pass && !added && !updated) {
    _fsExtra2.default.copySync(diffOutputPath, diffDotPath);
    _fsExtra2.default.removeSync(diffOutputPath);
    _fsExtra2.default.copySync(snapshotKebabPath, snapshotKebabPathFail);
    _fsExtra2.default.removeSync(snapshotKebabPath);
    snapshotResults.diffOutputPath = diffDotPath;

    console.log(`Screenshot was different from saved snapshot.\n  See diff for details: ${snapshotResults.diffOutputPath}`);

    return {
      path: diffDotPath
    };
  }

  _fsExtra2.default.copySync(snapshotKebabPath, snapshotDotPath);
  _fsExtra2.default.removeSync(snapshotKebabPath);
  snapshotResults.diffOutputPath = snapshotDotPath;

  return {
    path: snapshotDotPath
  };
}

function addMatchImageSnapshotPlugin(on) {
  on('task', {
    [_constants.MATCH]: matchImageSnapshotOptions,
    [_constants.RECORD]: matchImageSnapshotResults
  });
  on('after:screenshot', matchImageSnapshotPlugin);
}